import { memoizeOne } from '@bigcommerce/memoize';
import { FieldProps } from 'formik';
import React, { useCallback, useMemo, FunctionComponent, ChangeEvent } from 'react';

import { TranslatedString } from '../locale';
import { Fieldset, FormField, Label, Legend, TextInput } from '../ui/form';

const OrderComments: FunctionComponent = () => {
    const renderLabel = useCallback(name => (
        <Label hidden htmlFor={ name }>
            <TranslatedString id="shipping.order_comment_label" />
        </Label>
    ), []);

    const renderInput = useCallback(({ field }: FieldProps) => (
        <TextInput
            { ...field }
            autoComplete={ 'off' }
            maxLength={ 2000 }
        />
    ), []);

    const legend = useMemo(() => (
        <Legend>
            <TranslatedString id="shipping.order_comment_label" />
        </Legend>
    ), []);

    const handleChange = useCallback(memoizeOne((field: FieldProps['field'], form: FieldProps['form']) => {
        return (event: ChangeEvent<any>) => {
            // TODO: update set value
            form.setFieldValue(field.name, event.target.value);
        };
    }), []);

    const renderInputExtra = useCallback(({ field, form }: FieldProps) => (
        <TextInput
            { ...field }
            autoComplete={ 'off' }
            additionalClassName="shipping-extra-comment-input"
            onChange={ handleChange(field, form) }
        />
    ), []);

    return <>
        <Fieldset legend={ legend } testId="checkout-shipping-comments">
            <FormField
                input={ renderInput }
                label={ renderLabel }
                name="orderComment"
            />
        </Fieldset>
        <Fieldset additionalClassName="checkout-shipping-extra-comments" id="shipping-extra-comments">
            <FormField
                input={ renderInputExtra }
                label={
                    <Label htmlFor="shipping-customer-carrier">
                        Customer Carrier (UPS or Fedex Only)
                    </Label>
                }
                additionalClassName="shipping-customer-carrier"
                name="shippingCustomerCarrier"
            />
            <FormField
                input={ renderInputExtra }
                label={
                    <Label htmlFor="shipping-customer-account">
                        Customer Carrier Account Number
                    </Label>
                }
                additionalClassName="shipping-customer-account"
                name="shippingCustomerAccount"
            />
            <FormField
                input={ renderInputExtra }
                label={
                    <Label htmlFor="shipping-customer-phone">
                        Shipping Method (1 Day, 2nd Day, 3 Day Select)
                    </Label>
                }
                additionalClassName="shipping-customer-phone"
                name="shippingCustomerPhone"
            />
        </Fieldset>
    </>;
};

export default OrderComments;
